@@ -14,6 +14,7 @@ import stem.connection
 import stem.control
 import stem.descriptor.reader
 import stem.descriptor.router_status_entry
+import stem.directory
 import stem.response.protocolinfo
 import stem.socket
 import stem.util.str_tools
@@ -254,15 +255,50 @@ class TestController(unittest.TestCase):
       self.assertEqual({}, controller.get_info([]))
       self.assertEqual({}, controller.get_info([], {}))
 
+  @test.require.controller
+  def test_getinfo_freshrelaydescs(self):
+    """
+    Exercises 'GETINFO status/fresh-relay-descs'.
+    """
+
+    with test.runner.get_runner().get_tor_controller() as controller:
+      response = controller.get_info('status/fresh-relay-descs')
+      div = response.find('\nextra-info ')
+      nickname = controller.get_conf('Nickname')
+
+      if div == -1:
+        self.fail('GETINFO response should have both a server and extrainfo descriptor:\n%s' % response)
+
+      server_desc = stem.descriptor.server_descriptor.ServerDescriptor(response[:div], validate = True)
+      extrainfo_desc = stem.descriptor.extrainfo_descriptor.ExtraInfoDescriptor(response[div:], validate = True)
+
+      self.assertEqual(nickname, server_desc.nickname)
+      self.assertEqual(nickname, extrainfo_desc.nickname)
+      self.assertEqual(controller.get_info('address'), server_desc.address)
+      self.assertEqual(test.runner.ORPORT, server_desc.or_port)
+
+  @test.require.controller
+  @test.require.online
+  def test_getinfo_dir_status(self):
+    """
+    Exercise 'GETINFO dir/status-vote/*'.
+    """
+
+    with test.runner.get_runner().get_tor_controller() as controller:
+      consensus = controller.get_info('dir/status-vote/current/consensus')
+      self.assertTrue('moria1' in consensus, 'moria1 not found in the consensus')
+
+      if test.tor_version() >= stem.version.Version('0.4.3.1-alpha'):
+        microdescs = controller.get_info('dir/status-vote/current/consensus-microdesc')
+        self.assertTrue('moria1' in microdescs, 'moria1 not found in the microdescriptor consensus')
+
   @test.require.controller
   def test_get_version(self):
     """
     Test that the convenient method get_version() works.
     """
 
-    runner = test.runner.get_runner()
-
-    with runner.get_tor_controller() as controller:
+    with test.runner.get_runner().get_tor_controller() as controller:
       version = controller.get_version()
       self.assertTrue(isinstance(version, stem.version.Version))
       self.assertEqual(version, test.tor_version())
@@ -637,7 +673,7 @@ class TestController(unittest.TestCase):
     runner = test.runner.get_runner()
 
     with runner.get_tor_controller() as controller:
-      response = controller.create_ephemeral_hidden_service(4567, basic_auth = {'alice': 'nKwfvVPmTNr2k2pG0pzV4g', 'bob': None})
+      response = controller.create_ephemeral_hidden_service(4567, key_content = 'RSA1024', basic_auth = {'alice': 'nKwfvVPmTNr2k2pG0pzV4g', 'bob': None})
       self.assertEqual([response.service_id], controller.list_ephemeral_hidden_services())
       self.assertTrue(response.private_key is not None)
       self.assertEqual(['bob'], list(response.client_auth.keys()))  # newly created credentials were only created for bob
@@ -773,6 +809,7 @@ class TestController(unittest.TestCase):
         self.assertEqual('17234 127.0.0.1:17235', controller.get_conf('HiddenServicePort'))
       finally:
         # reverts configuration changes
+
         controller.set_options((
           ('ExitPolicy', 'reject *:*'),
           ('ConnLimit', None),
@@ -783,6 +820,29 @@ class TestController(unittest.TestCase):
 
         shutil.rmtree(tmpdir)
 
+  @test.require.controller
+  def test_set_conf_for_usebridges(self):
+    """
+    Ensure we can set UseBridges=1 and also set a Bridge. This is a tor
+    regression check (:trac:`31945`).
+    """
+
+    with test.runner.get_runner().get_tor_controller() as controller:
+      orport = controller.get_conf('ORPort')
+
+      try:
+        controller.set_conf('ORPort', '0')  # ensure we're not a relay so UseBridges is usabe
+        controller.set_options([('UseBridges', '1'), ('Bridge', '127.0.0.1:9999')])
+        self.assertEqual('127.0.0.1:9999', controller.get_conf('Bridge'))
+      finally:
+        # reverts configuration changes
+
+        controller.set_options((
+          ('ORPort', orport),
+          ('UseBridges', None),
+          ('Bridge', None),
+        ), reset = True)
+
   @test.require.controller
   def test_set_conf_when_immutable(self):
     """
@@ -1069,15 +1129,15 @@ class TestController(unittest.TestCase):
         # Make sure we have the stream for which we asked, otherwise
         # the next assertion would be a false positive.
 
-        self.assertEqual([built_stream.id], [stream.id for stream in controller.get_streams()])
+        self.assertTrue(built_stream.id in [stream.id for stream in controller.get_streams()])
 
         # Try to close our stream...
 
         controller.close_stream(built_stream.id)
 
-        # ...which means there are zero streams.
+        # ... after which the stream should no longer be present.
 
-        self.assertEqual([], controller.get_streams())
+        self.assertFalse(built_stream.id in [stream.id for stream in controller.get_streams()])
 
       # unknown stream
 
@@ -1122,6 +1182,84 @@ class TestController(unittest.TestCase):
       ip_addr = response[response.find(b'\r\n\r\n'):].strip()
       self.assertTrue(stem.util.connection.is_valid_ipv4_address(stem.util.str_tools._to_unicode(ip_addr)), "'%s' isn't an address" % ip_addr)
 
+  @test.require.controller
+  def test_mapaddress_offline(self):
+    runner = test.runner.get_runner()
+
+    with runner.get_tor_controller() as controller:
+      # try mapping one element, ensuring results are as expected
+
+      map1 = {'1.2.1.2': 'ifconfig.me'}
+      x = controller.map_address(map1)
+      self.assertEqual(x, map1)
+
+      # try mapping two elements, ensuring results are as expected
+
+      map2 = {'1.2.3.4': 'foobar.example.com',
+              '1.2.3.5': 'barfuzz.example.com'}
+
+      x = controller.map_address(map2)
+      self.assertEqual(x, map2)
+
+      # try mapping zero elements
+
+      self.assertRaises(stem.InvalidRequest, controller.map_address, {})
+
+      # try a virtual mapping to IPv4, the default virtualaddressrange is 127.192.0.0/10
+
+      map3 = {'0.0.0.0': 'quux'}
+      x = controller.map_address(map3)
+      self.assertEquals(len(x), 1)
+      addr1, target = list(x.items())[0]
+
+      self.assertTrue(addr1.startswith('127.'), '%s did not start with 127.' % addr1)
+      self.assertEquals(target, 'quux')
+
+      # try a virtual mapping to IPv6, the default IPv6 virtualaddressrange is FE80::/10
+
+      map4 = {'::': 'quibble'}
+      x = controller.map_address(map4)
+      self.assertEquals(len(x), 1)
+      addr2, target = list(x.items())[0]
+
+      self.assertTrue(addr2.startswith('[fe'), '%s did not start with [fe.' % addr2)
+      self.assertEquals(target, 'quibble')
+
+      def address_mappings(addr_type):
+        response = controller.get_info(['address-mappings/%s' % addr_type])
+        result = {}
+
+        for line in response['address-mappings/%s' % addr_type].splitlines():
+          k, v, timeout = line.split()
+          result[k] = v
+
+        return result
+
+      # ask for a list of all the address mappings we've added
+
+      self.assertEquals({
+        '1.2.1.2': 'ifconfig.me',
+        '1.2.3.4': 'foobar.example.com',
+        '1.2.3.5': 'barfuzz.example.com',
+        addr1: 'quux',
+        addr2: 'quibble',
+      }, address_mappings('control'))
+
+      # ask for a list of all the address mappings
+
+      self.assertEquals({
+        '1.2.1.2': 'ifconfig.me',
+        '1.2.3.4': 'foobar.example.com',
+        '1.2.3.5': 'barfuzz.example.com',
+        addr1: 'quux',
+        addr2: 'quibble',
+      }, address_mappings('all'))
+
+      # Now ask for a list of only the mappings configured with the
+      # configuration.  Ours should not be there.
+
+      self.assertEquals({}, address_mappings('config'))
+
   @test.require.controller
   @test.require.online
   @test.require.version(Requirement.MICRODESCRIPTOR_IS_DEFAULT)
@@ -1356,7 +1494,6 @@ class TestController(unittest.TestCase):
       circuits = controller.get_circuits()
       self.assertTrue(new_circ in [circ.id for circ in circuits])
 
-  @test.require.online
   @test.require.controller
   def test_transition_to_relay(self):
     """
