@@ -1,4 +1,4 @@
-# Copyright 2012-2018, Damian Johnson and The Tor Project
+# Copyright 2012-2019, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -12,7 +12,7 @@ run.
   |- needs - skips the test unless a requirement is met
   |
   |- cryptography - skips test unless the cryptography module is present
-  |- pynacl - skips test unless the pynacl module is present
+  |- ed25519_support - skips test unless cryptography has ed25519 support
   |- command - requires a command to be on the path
   |- proc - requires the platform to have recognized /proc contents
   |
@@ -49,7 +49,7 @@ def only_run_once(func):
 
 def needs(condition, message):
   """
-  Skips teh test unless the conditional evaluates to 'true'.
+  Skips the test unless the conditional evaluates to 'true'.
   """
 
   def decorator(func):
@@ -99,7 +99,7 @@ def version(req_version):
 
 
 cryptography = needs(stem.prereq.is_crypto_available, 'requires cryptography')
-pynacl = needs(stem.prereq._is_pynacl_available, 'requires pynacl module')
+ed25519_support = needs(lambda: stem.prereq.is_crypto_available(ed25519 = True), 'requires ed25519 support')
 proc = needs(stem.util.proc.is_available, 'proc unavailable')
 controller = needs(_can_access_controller, 'no connection')
 ptrace = needs(_can_ptrace, 'DisableDebuggerAttachment is set')
