@@ -1,4 +1,4 @@
-# Copyright 2012-2018, Damian Johnson and The Tor Project
+# Copyright 2012-2019, Damian Johnson and The Tor Project
 # See LICENSE for licensing information
 
 """
@@ -8,15 +8,17 @@
 
   Initialization Tasks
   |- STEM_VERSION - checks our stem version
+  |- STEM_COMMIT - checks our stem commit
   |- TOR_VERSION - checks our tor version
+  |- TOR_COMMIT - checks our tor commit
   |- PYTHON_VERSION - checks our python version
   |- PLATFORM_VERSION - checks our operating system version
   |- CRYPTO_VERSION - checks our version of cryptography
-  |- PYNACL_VERSION - checks our version of pynacl
   |- MOCK_VERSION - checks our version of mock
   |- PYFLAKES_VERSION - checks our version of pyflakes
   |- PYCODESTYLE_VERSION - checks our version of pycodestyle
   |- CLEAN_PYC - removes any *.pyc without a corresponding *.py
+  |- REMOVE_TOR_DATA_DIR - removes our tor data directory
   |- IMPORT_TESTS - ensure all test modules have been imported
   |- UNUSED_TESTS - checks to see if any tests are missing from our settings
   |- PYFLAKES_TASK - static checks
@@ -26,6 +28,7 @@
 import os
 import platform
 import re
+import shutil
 import sys
 import time
 import traceback
@@ -41,6 +44,8 @@ import test.output
 
 from test.output import STATUS, ERROR, NO_NL, println
 
+TASK_DESCRIPTION_WIDTH = 40
+
 try:
   # TODO: remove check when dropping python 2.6 support
 
@@ -67,12 +72,49 @@ SRC_PATHS = [os.path.join(test.STEM_BASE, path) for path in (
   os.path.join('docs', 'roles.py'),
 )]
 
-PYFLAKES_UNAVAILABLE = 'Static error checking requires pyflakes version 0.7.3 or later. Please install it from ...\n  http://pypi.python.org/pypi/pyflakes\n'
-PYCODESTYLE_UNAVAILABLE = 'Style checks require pycodestyle version 1.4.2 or later. Please install it from...\n  http://pypi.python.org/pypi/pycodestyle\n'
+PYFLAKES_UNAVAILABLE = 'Static error checking requires pyflakes version 0.7.3 or later. Please install it from ...\n  https://pypi.org/project/pyflakes/\n'
+PYCODESTYLE_UNAVAILABLE = 'Style checks require pycodestyle version 1.4.2 or later. Please install it from...\n  https://pypi.org/project/pycodestyle/\n'
+
+
+def _check_stem_version():
+  commit = _git_commit(os.path.join(test.STEM_BASE, '.git'))
+
+  if commit:
+    return '%s (commit %s)' % (stem.__version__, commit[:8])
+  else:
+    return stem.__version__
 
 
 def _check_tor_version(tor_path):
-  return str(test.tor_version(tor_path)).split()[0]
+  version = test.tor_version(tor_path)
+  version_str = str(version).split()[0]
+
+  if version.git_commit:
+    return '%s (commit %s)' % (version_str, version.git_commit[:8])
+  else:
+    return version_str
+
+
+def _check_python_version():
+  interpreter = platform.python_implementation()
+  version = platform.python_version()
+
+  return version if interpreter == 'CPython' else '%s (%s)' % (interpreter, version)
+
+
+def _git_commit(git_dir):
+  if not stem.util.system.is_available('git'):
+    return None
+  elif not os.path.exists(git_dir):
+    return None
+
+  cmd = ['git', '--git-dir', git_dir, 'rev-parse', 'HEAD']
+  git_output = stem.util.system.call(cmd)
+
+  if len(git_output) != 1:
+    raise ValueError("Expected a single line from '%s':\n\n%s" % (' '.join(cmd), git_output))
+  else:
+    return git_output[0]
 
 
 def _check_platform_version():
@@ -80,7 +122,10 @@ def _check_platform_version():
     extra = platform.release()
   elif platform.system() == 'Darwin':
     extra = platform.release()
-  elif platform.system() == 'Linux':
+  elif platform.system() == 'Linux' and hasattr(platform, 'linux_distribution'):
+    # TODO: platform.linux_distribution() was removed in python 3.8
+    # https://trac.torproject.org/projects/tor/ticket/30847
+
     extra = ' '.join(platform.linux_distribution()[:2])
   else:
     extra = None
@@ -98,6 +143,20 @@ def _clean_orphaned_pyc(paths):
   return ['removed %s' % path for path in stem.util.test_tools.clean_orphaned_pyc(paths)]
 
 
+def _remove_tor_data_dir():
+  """
+  Empties tor's data directory.
+  """
+
+  config_test_dir = CONFIG['integ.test_directory']
+
+  if config_test_dir and os.path.exists(config_test_dir):
+    shutil.rmtree(config_test_dir, ignore_errors = True)
+    return 'done'
+  else:
+    return 'skipped'
+
+
 def _import_tests():
   """
   Ensure all tests have been imported. This is important so tests can
@@ -137,7 +196,7 @@ def _check_for_unused_tests(paths):
       with open(py_path) as f:
         file_contents = f.read()
 
-      test_match = re.search('^class (\S*)\(unittest.TestCase\):$', file_contents, re.MULTILINE)
+      test_match = re.search('^class (\\S*)\\(unittest.TestCase\\):$', file_contents, re.MULTILINE)
 
       if test_match:
         class_name = test_match.groups()[0]
@@ -202,7 +261,7 @@ class Task(object):
     start_time = time.time()
     println('  %s...' % self.label, STATUS, NO_NL)
 
-    padding = 50 - len(self.label)
+    padding = TASK_DESCRIPTION_WIDTH - len(self.label)
     println(' ' * padding, NO_NL)
 
     try:
@@ -214,7 +273,7 @@ class Task(object):
       self.is_successful = True
       output_msg = 'running' if self._is_background_task else 'done'
 
-      if self.result and self.print_result and isinstance(self.result, str):
+      if self.result and self.print_result and stem.util._is_str(self.result):
         output_msg = self.result
       elif self.print_runtime:
         output_msg += ' (%0.1fs)' % (time.time() - start_time)
@@ -265,20 +324,20 @@ class StaticCheckTask(Task):
       return super(StaticCheckTask, self).run()
     else:
       println('  %s...' % self.label, STATUS, NO_NL)
-      println(' ' * (50 - len(self.label)), NO_NL)
+      println(' ' * (TASK_DESCRIPTION_WIDTH - len(self.label)), NO_NL)
       println('unavailable', STATUS)
 
 
-STEM_VERSION = Task('stem version', lambda: stem.__version__)
+STEM_VERSION = Task('stem version', _check_stem_version)
 TOR_VERSION = Task('tor version', _check_tor_version)
-PYTHON_VERSION = Task('python version', lambda: platform.python_version())
-PLATFORM_VERSION = Task('operating system version', _check_platform_version)
+PYTHON_VERSION = Task('python version', _check_python_version)
+PLATFORM_VERSION = Task('operating system', _check_platform_version)
 CRYPTO_VERSION = ModuleVersion('cryptography version', 'cryptography', stem.prereq.is_crypto_available)
-PYNACL_VERSION = ModuleVersion('pynacl version', 'nacl', stem.prereq._is_pynacl_available)
 MOCK_VERSION = ModuleVersion('mock version', ['unittest.mock', 'mock'], stem.prereq.is_mock_available)
 PYFLAKES_VERSION = ModuleVersion('pyflakes version', 'pyflakes')
 PYCODESTYLE_VERSION = ModuleVersion('pycodestyle version', ['pycodestyle', 'pep8'])
 CLEAN_PYC = Task('checking for orphaned .pyc files', _clean_orphaned_pyc, (SRC_PATHS,), print_runtime = True)
+REMOVE_TOR_DATA_DIR = Task('emptying our tor data directory', _remove_tor_data_dir)
 IMPORT_TESTS = Task('importing test modules', _import_tests, print_runtime = True)
 
 UNUSED_TESTS = Task('checking for unused tests', _check_for_unused_tests, [(
