@@ -562,6 +562,34 @@ class TestControl(unittest.TestCase):
 
     self.assertEqual(432, self.controller.get_pid())
 
+  @patch('stem.control.Controller.get_version', Mock(return_value = stem.version.Version('0.5.0.14')))
+  @patch('stem.socket.ControlSocket.is_localhost', Mock(return_value = False))
+  @patch('stem.control.Controller.get_info')
+  @patch('time.time', Mock(return_value = 1000.0))
+  def test_get_uptime_by_getinfo(self, getinfo_mock):
+    """
+    Exercise the get_uptime() resolution via a GETINFO query.
+    """
+
+    getinfo_mock.return_value = '321'
+    self.assertEqual(321.0, self.controller.get_uptime())
+    self.controller.clear_cache()
+
+    getinfo_mock.return_value = 'abc'
+    self.assertRaisesWith(ValueError, "'GETINFO uptime' did not provide a valid numeric response: abc", self.controller.get_uptime)
+
+  @patch('stem.socket.ControlSocket.is_localhost', Mock(return_value = True))
+  @patch('stem.control.Controller.get_version', Mock(return_value = stem.version.Version('0.1.0.14')))
+  @patch('stem.control.Controller.get_pid', Mock(return_value = '12'))
+  @patch('stem.util.system.start_time', Mock(return_value = 5000.0))
+  @patch('time.time', Mock(return_value = 5200.0))
+  def test_get_uptime_by_process(self):
+    """
+    Exercise the get_uptime() resolution via process age.
+    """
+
+    self.assertEqual(200.0, self.controller.get_uptime())
+
   @patch('stem.control.Controller.get_info')
   def test_get_network_status_for_ourselves(self, get_info_mock):
     """
@@ -669,7 +697,6 @@ class TestControl(unittest.TestCase):
 
     self.assertRaises(InvalidRequest, self.controller.add_event_listener, Mock(), EventType.SIGNAL)
 
-  @patch('time.time', Mock(return_value = TEST_TIMESTAMP))
   def test_events_get_received(self):
     """
     Trigger an event, checking that our listeners get notified.
@@ -683,7 +710,6 @@ class TestControl(unittest.TestCase):
     self._emit_event(BW_EVENT)
     self.bw_listener.assert_called_once_with(BW_EVENT)
 
-  @patch('time.time', Mock(return_value = TEST_TIMESTAMP))
   def test_event_listing_with_error(self):
     """
     Raise an exception in an event listener to confirm it doesn't break our
@@ -700,7 +726,6 @@ class TestControl(unittest.TestCase):
     self._emit_event(BW_EVENT)
     self.bw_listener.assert_called_once_with(BW_EVENT)
 
-  @patch('time.time', Mock(return_value = TEST_TIMESTAMP))
   def test_event_listing_with_malformed_event(self):
     """
     Attempt to parse a malformed event emitted from Tor. It's important this
@@ -857,17 +882,19 @@ class TestControl(unittest.TestCase):
     #      processed asynchronously, so the only way to endsure it's done
     #      with its work is to join on the thread.
 
-    with patch('stem.control.Controller.is_alive') as is_alive_mock:
-      is_alive_mock.return_value = True
-      self.controller._launch_threads()
+    with patch('time.time', Mock(return_value = TEST_TIMESTAMP)):
+      with patch('stem.control.Controller.is_alive') as is_alive_mock:
+        is_alive_mock.return_value = True
+        self.controller._launch_threads()
 
-      try:
-        # Converting an event back into an uncast ControlMessage, then feeding it
-        # into our controller's event queue.
+        try:
+          # Converting an event back into an uncast ControlMessage, then feeding it
+          # into our controller's event queue.
 
-        uncast_event = ControlMessage.from_str(event.raw_content())
-        self.controller._event_queue.put(uncast_event)
-        self.controller._event_notice.set()
-      finally:
-        is_alive_mock.return_value = False
-        self.controller._close()
+          uncast_event = ControlMessage.from_str(event.raw_content())
+          self.controller._event_queue.put(uncast_event)
+          self.controller._event_notice.set()
+          self.controller._event_queue.join()  # block until the event is consumed
+        finally:
+          is_alive_mock.return_value = False
+          self.controller._close()
