@@ -2,19 +2,30 @@
 Unit tests for the stem.util.connection functions.
 """
 
+import io
 import platform
 import unittest
 
+import stem
 import stem.util.connection
 
 from stem.util.connection import Resolver, Connection
 
+try:
+  # account for urllib's change between python 2.x and 3.x
+  import urllib.request as urllib
+except ImportError:
+  import urllib2 as urllib
+
 try:
   # added in python 3.3
   from unittest.mock import Mock, patch
 except ImportError:
   from mock import Mock, patch
 
+URL_OPEN = 'urllib.request.urlopen' if stem.prereq.is_python_3() else 'urllib2.urlopen'
+URL = 'https://example.unit.test.url'
+
 NETSTAT_OUTPUT = """\
 Active Internet connections (w/o servers)
 Proto Recv-Q Send-Q Local Address           Foreign Address         State       PID/Program name
@@ -166,6 +177,39 @@ _tor     tor        15843   20* internet stream tcp 0x0 192.168.1.100:36174 -->
 
 
 class TestConnection(unittest.TestCase):
+  @patch(URL_OPEN)
+  def test_download(self, urlopen_mock):
+    urlopen_mock.return_value = io.BytesIO(b'hello')
+
+    self.assertEqual(b'hello', stem.util.connection.download(URL))
+    urlopen_mock.assert_called_with(URL, timeout = None)
+
+  @patch(URL_OPEN)
+  def test_download_failure(self, urlopen_mock):
+    urlopen_mock.side_effect = urllib.URLError('boom')
+
+    try:
+      stem.util.connection.download(URL)
+      self.fail('expected a stem.DownloadFailed to be raised')
+    except stem.DownloadFailed as exc:
+      self.assertEqual('Failed to download from https://example.unit.test.url (URLError): boom', str(exc))
+      self.assertEqual(URL, exc.url)
+      self.assertEqual('boom', exc.error.reason)
+      self.assertEqual(urllib.URLError, type(exc.error))
+      self.assertTrue('return urllib.urlopen(url, timeout = timeout).read()' in exc.stacktrace_str)
+
+  @patch(URL_OPEN)
+  def test_download_retries(self, urlopen_mock):
+    urlopen_mock.side_effect = urllib.URLError('boom')
+
+    self.assertRaisesRegexp(IOError, 'boom', stem.util.connection.download, URL)
+    self.assertEqual(1, urlopen_mock.call_count)
+
+    urlopen_mock.reset_mock()
+
+    self.assertRaisesRegexp(IOError, 'boom', stem.util.connection.download, URL, retries = 4)
+    self.assertEqual(5, urlopen_mock.call_count)
+
   @patch('os.access')
   @patch('stem.util.system.is_available')
   @patch('stem.util.proc.is_available')
